# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from typing import Any, Callable, Optional, TypeVar

from azure.core import PipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict

from .._configuration import NotVersionedClientConfiguration
from .._utils.serialization import Serializer
from .._utils.utils import ClientMixinABC

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_not_versioned_without_api_version_request(**kwargs: Any) -> HttpRequest:  # pylint: disable=name-too-long
    # Construct URL
    _url = "/server/versions/not-versioned/without-api-version"

    return HttpRequest(method="HEAD", url=_url, **kwargs)


def build_not_versioned_with_query_api_version_request(  # pylint: disable=name-too-long
    *, api_version: str, **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/server/versions/not-versioned/with-query-api-version"

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    return HttpRequest(method="HEAD", url=_url, params=_params, **kwargs)


def build_not_versioned_with_path_api_version_request(  # pylint: disable=name-too-long
    api_version: str, **kwargs: Any
) -> HttpRequest:
    # Construct URL
    _url = "/server/versions/not-versioned/with-path-api-version/{apiVersion}"
    path_format_arguments = {
        "apiVersion": _SERIALIZER.url("api_version", api_version, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    return HttpRequest(method="HEAD", url=_url, **kwargs)


class _NotVersionedClientOperationsMixin(
    ClientMixinABC[PipelineClient[HttpRequest, HttpResponse], NotVersionedClientConfiguration]
):

    @distributed_trace
    def without_api_version(self, **kwargs: Any) -> bool:
        """without_api_version.

        :return: bool
        :rtype: bool
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_not_versioned_without_api_version_request(
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore
        return 200 <= response.status_code <= 299

    @distributed_trace
    def with_query_api_version(self, *, api_version: str, **kwargs: Any) -> bool:
        """with_query_api_version.

        :keyword api_version: Required.
        :paramtype api_version: str
        :return: bool
        :rtype: bool
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_not_versioned_with_query_api_version_request(
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore
        return 200 <= response.status_code <= 299

    @distributed_trace
    def with_path_api_version(self, api_version: str, **kwargs: Any) -> bool:
        """with_path_api_version.

        :param api_version: Required.
        :type api_version: str
        :return: bool
        :rtype: bool
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_not_versioned_with_path_api_version_request(
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore
        return 200 <= response.status_code <= 299



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
{%- for mod_api_version in default_models %}
from .{{ mod_api_version }}.models import *
{%- endfor %}



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_import_project.py
DESCRIPTION:
    This sample demonstrates how to import assets into a **Text Authoring** project.
USAGE:
    python sample_import_project.py
REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_TEXT_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET
NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_TEXT_ENDPOINT
      - AZURE_TEXT_KEY
OPTIONAL ENV VARS:
    PROJECT_NAME                 # defaults to "<project-name>"
    LANGUAGE_TAG                 # defaults to "<language-tag>"
    STORAGE_INPUT_CONTAINER_NAME # defaults to "<storage-container-name>"
    DOC1_PATH                    # defaults to "<doc1-path>"
    DOC2_PATH                    # defaults to "<doc2-path>"
"""

# [START text_authoring_import_project]
import os
from azure.identity import DefaultAzureCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.textanalytics.authoring import TextAuthoringClient
from azure.ai.textanalytics.authoring.models import (
    CreateProjectOptions,
    ExportedProject,
    ProjectSettings,
    ExportedCustomSingleLabelClassificationProjectAsset,
    ExportedCustomSingleLabelClassificationDocument,
    ExportedDocumentClass,
    ExportedClass,
    ProjectKind,
    StringIndexType,
)


def sample_import_project():
    # settings
    endpoint = os.environ["AZURE_TEXT_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    language_tag = os.environ.get("LANGUAGE_TAG", "<language-tag>")  # e.g., "en"
    storage_container = os.environ.get("STORAGE_INPUT_CONTAINER_NAME", "<storage-container-name>")
    doc1_path = os.environ.get("DOC1_PATH", "<doc1-path>")  # e.g., "01.txt"
    doc2_path = os.environ.get("DOC2_PATH", "<doc2-path>")  # e.g., "02.txt"

    # create a client with AAD
    credential = DefaultAzureCredential()
    client = TextAuthoringClient(endpoint, credential=credential)

    # project-scoped client
    project_client = client.get_project_client(project_name)

    # ---------- Arrange metadata (project definition) ----------
    project_metadata = CreateProjectOptions(
        project_kind=ProjectKind.CUSTOM_SINGLE_LABEL_CLASSIFICATION,
        storage_input_container_name=storage_container,
        project_name=project_name,
        language=language_tag,
        description="Sample project imported via Python SDK.",
        multilingual=False,
        settings=ProjectSettings(),
    )

    # ---------- Arrange assets (classes + labeled documents) ----------
    project_assets = ExportedCustomSingleLabelClassificationProjectAsset(
        classes=[
            ExportedClass(category="ClassA"),
            ExportedClass(category="ClassB"),
            ExportedClass(category="ClassC"),
        ],
        documents=[
            ExportedCustomSingleLabelClassificationDocument(
                document_class=ExportedDocumentClass(category="ClassA"),
                location=doc1_path,
                language=language_tag,
            ),
            ExportedCustomSingleLabelClassificationDocument(
                document_class=ExportedDocumentClass(category="ClassB"),
                location=doc2_path,
                language=language_tag,
            ),
        ],
    )

    exported_project = ExportedProject(
        project_file_version="2022-05-01",
        string_index_type=StringIndexType.UTF16_CODE_UNIT,
        metadata=project_metadata,
        assets=project_assets,
    )

    # ---------- Import (LRO) with error handling ----------
    poller = project_client.project.begin_import(body=exported_project)
    try:
        poller.result()  # completes with None; raises on failure
        print("Import completed.")
        print(f"done: {poller.done()}")
        print(f"status: {poller.status()}")
    except HttpResponseError as e:
        print(f"Operation failed: {e.message}")
        print(e.error)


# [END text_authoring_import_project]


def main():
    sample_import_project()


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Root conftest — ensures the project root is on sys.path so that
``from tests._helpers import …`` works regardless of how pytest is invoked."""

import sys
from pathlib import Path

_PROJECT_ROOT = str(Path(__file__).resolve().parent.parent)
if _PROJECT_ROOT not in sys.path:
    sys.path.insert(0, _PROJECT_ROOT)



# coding=utf-8

from .._utils.model_base import Model as _Model


class EmptyInput(_Model):
    """Empty model used in operation parameters."""


class EmptyInputOutput(_Model):
    """Empty model used in both parameter and return type."""


class EmptyOutput(_Model):
    """Empty model used in operation return type."""



"""
Helper functions for updating conda files.
"""

import os
import glob
from functools import lru_cache
from typing import Optional
import csv
from ci_tools.logging import logger
import urllib.request
from datetime import datetime
from ci_tools.parsing import ParsedSetup
from packaging.version import Version
from pypi_tools.pypi import PyPIClient, retrieve_versions_from_pypi


ROOT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
SDK_DIR = os.path.join(ROOT_DIR, "sdk")

AZURE_SDK_CSV_URL = "https://raw.githubusercontent.com/Azure/azure-sdk/main/_data/releases/latest/python-packages.csv"
PACKAGE_COL = "Package"
LATEST_GA_DATE_COL = "LatestGADate"
VERSION_GA_COL = "VersionGA"
FIRST_GA_DATE_COL = "FirstGADate"
DISPLAY_NAME_COL = "DisplayName"
SERVICE_NAME_COL = "ServiceName"
REPO_PATH_COL = "RepoPath"
TYPE_COL = "Type"
SUPPORT_COL = "Support"

# =====================================
# Helpers for handling bundled releases
# =====================================


@lru_cache(maxsize=None)
def _build_package_path_index() -> dict[str, str]:
    """
    Build a one-time index mapping package names to their filesystem paths.

    This scans the sdk/ directory once and caches the result for all subsequent lookups.
    """
    all_paths = glob.glob(os.path.join(SDK_DIR, "*", "*"))
    # Exclude temp directories like .tox, .venv, __pycache__, etc.
    return {
        os.path.basename(p): p
        for p in all_paths
        if os.path.isdir(p) and not os.path.basename(p).startswith((".", "__"))
    }


def get_package_path(package_name: str) -> Optional[str]:
    """Get the filesystem path of an SDK package given its name."""
    path_index = _build_package_path_index()
    package_path = path_index.get(package_name)
    if not package_path:
        logger.warning(f"Package path not found for package: {package_name}")
        return None
    return package_path


def get_bundle_name(package_name: str) -> Optional[str]:
    """
    Check bundled release config from package's pyproject.toml file given the package name.

    If bundled, return the bundle name; otherwise, return None.
    """
    package_path = get_package_path(package_name)
    if not package_path:
        logger.warning(f"Cannot determine package path for {package_name}")
        return None
    parsed = ParsedSetup.from_path(package_path)
    if not parsed:
        # can't proceed, need to know if it's bundled or not
        logger.error(f"Failed to parse setup for package {package_name}")
        raise Exception(f"Failed to parse setup for package {package_name}")

    conda_config = parsed.get_conda_config()

    if not conda_config:
        if is_stable_on_pypi(package_name):
            raise Exception(
                f"Stable release package {package_name} needs a conda config"
            )

        logger.warning(
            f"No conda config found for package {package_name}, which may be a pre-release"
        )
        return None

    if conda_config and "bundle_name" in conda_config:
        return conda_config["bundle_name"]

    return None


def map_bundle_to_packages(
    package_names: list[str],
) -> tuple[dict[str, list[str]], list[str]]:
    """Create a mapping of bundle names to their constituent package names.

    :return: Tuple of (bundle_map, failed_packages) where failed_packages are packages that threw exceptions.
    """
    logger.info("Mapping bundle names to packages...")

    bundle_map = {}
    failed_packages = []
    for package_name in package_names:
        logger.debug(f"Processing package for bundle mapping: {package_name}")
        try:
            bundle_name = get_bundle_name(package_name)
            if bundle_name:
                logger.debug(f"Bundle name for package {package_name}: {bundle_name}")
                bundle_map.setdefault(bundle_name, []).append(package_name)
        except Exception as e:
            logger.error(f"Failed to get bundle name for {package_name}: {e}")
            failed_packages.append(package_name)
            continue

    return bundle_map, failed_packages


# =====================================
# Utility functions for parsing data
# =====================================


def parse_csv() -> list[dict[str, str]]:
    """Download and parse the Azure SDK Python packages CSV file."""
    try:
        logger.info(f"Downloading CSV from {AZURE_SDK_CSV_URL}")

        with urllib.request.urlopen(AZURE_SDK_CSV_URL, timeout=10) as response:
            csv_content = response.read().decode("utf-8")

        # Parse the CSV content
        csv_reader = csv.DictReader(csv_content.splitlines())
        packages = list(csv_reader)

        logger.info(f"Successfully parsed {len(packages)} packages from CSV")

        return packages

    except Exception as e:
        logger.error(f"Failed to download or parse CSV: {e}")
        return []


def is_mgmt_package(pkg: dict[str, str]) -> bool:
    pkg_name = pkg.get(PACKAGE_COL, "")
    _type = pkg.get(TYPE_COL, "")
    if _type == "mgmt":
        return True
    elif _type == "client":
        return False
    else:
        return pkg_name != "azure-mgmt-core" and (
            "mgmt" in pkg_name or "cognitiveservices" in pkg_name
        )


def separate_packages_by_type(
    packages: list[dict[str, str]],
) -> tuple[list[dict[str, str]], list[dict[str, str]]]:
    """Separate packages into data plane and management plane libraries."""
    data_plane_packages = []
    mgmt_plane_packages = []

    for pkg in packages:
        if is_mgmt_package(pkg):
            mgmt_plane_packages.append(pkg)
        else:
            data_plane_packages.append(pkg)

    logger.debug(
        f"Separated {len(data_plane_packages)} data plane and {len(mgmt_plane_packages)} management plane packages"
    )

    return (data_plane_packages, mgmt_plane_packages)


def package_needs_update(
    package_row: dict[str, str], prev_release_date: str, is_new=False
) -> bool:
    """
    Check if the package is new or needs version update (i.e., FirstGADate or LatestGADate is after the last release).

    :param package_row: The parsed CSV row for the package.
    :param prev_release_date: The date of the previous release in "mm/dd/yyyy" format.
    :param is_new: Whether to check for new package (FirstGADate) or outdated package (LatestGADate).
    :return: if the package is new or needs an update.
    """
    compare_date = (
        package_row.get(FIRST_GA_DATE_COL)
        if is_new
        else package_row.get(LATEST_GA_DATE_COL)
    )

    logger.debug(
        f"Checking {'new package' if is_new else 'outdated package'} for package {package_row.get(PACKAGE_COL)} with against date: {compare_date}"
    )

    if not compare_date:
        logger.debug(
            f"Package {package_row.get(PACKAGE_COL)} is skipped due to missing {FIRST_GA_DATE_COL if is_new else LATEST_GA_DATE_COL}."
        )

        return False

    try:
        # Convert string dates to datetime objects for proper comparison
        compare_date = datetime.strptime(compare_date, "%m/%d/%Y")
        prev_date = datetime.strptime(prev_release_date, "%m/%d/%Y")
        logger.debug(
            f"Comparing {package_row.get(PACKAGE_COL)} CompareDate {compare_date} with previous release date {prev_date}"
        )
        return compare_date > prev_date
    except ValueError as e:
        logger.error(
            f"Date parsing error for package {package_row.get(PACKAGE_COL)}: {e}"
        )
        return False


def is_stable_on_pypi(package_name: str) -> bool:
    """
    Check if a package has any stable (GA) release on PyPI.

    :param package_name: The name of the package to check.
    :return: True if any stable version exists on PyPI, False otherwise.
    """
    try:
        versions = retrieve_versions_from_pypi(package_name)
        if not versions:
            logger.warning(f"No versions found on PyPI for {package_name}")
            return False

        # Check if any version is stable (not a prerelease)
        for v in versions:
            if not Version(v).is_prerelease:
                logger.debug(f"Package {package_name} has stable version {v}")
                return True

        logger.debug(f"Package {package_name} has no stable versions")
        return False

    except Exception as e:
        logger.warning(f"Failed to check PyPI for {package_name}: {e}")
        return False


def get_package_data_from_pypi(
    package_name: str,
) -> tuple[Optional[str], Optional[str]]:
    """Fetch the latest version and download URI for a package from PyPI."""
    try:
        client = PyPIClient()
        data = client.project(package_name)

        # Get the latest version
        latest_version = data["info"]["version"]
        if latest_version in data["releases"] and data["releases"][latest_version]:
            # Get the source distribution (sdist) if available
            files = data["releases"][latest_version]
            source_dist = next((f for f in files if f["packagetype"] == "sdist"), None)
            if source_dist:
                download_url = source_dist["url"]
                logger.info(
                    f"Found download URL for {package_name}=={latest_version}: {download_url}"
                )
                return latest_version, download_url

    except Exception as e:
        logger.error(f"Failed to fetch download URI from PyPI for {package_name}: {e}")
    return None, None


def build_package_index(conda_artifacts: list[dict]) -> dict[str, tuple[int, int]]:
    """Build an index of package name -> (artifact_idx, checkout_idx) for fast lookups in conda-sdk-client.yml."""
    package_index = {}

    for artifact_idx, artifact in enumerate(conda_artifacts):
        if "checkout" in artifact:
            for checkout_idx, checkout_item in enumerate(artifact["checkout"]):
                package_name = checkout_item.get("package")
                if package_name:
                    package_index[package_name] = (artifact_idx, checkout_idx)
    return package_index


def get_valid_package_imports(package_name: str) -> list[str]:
    """
    Inspect the package's actual module structure and return only valid imports.

    :param package_name: The name of the package (e.g., "azure-mgmt-advisor" or "azure-eventgrid").
    :return: List of valid module names for import (e.g., ["azure.eventgrid", "azure.eventgrid.aio"]).
    """
    package_path = get_package_path(package_name)
    if not package_path:
        logger.warning(
            f"Could not find package path for {package_name} to determine imports, using fallback"
        )
        return [package_name.replace("-", ".")]
    else:
        parsed = ParsedSetup.from_path(package_path)
        if not parsed or not parsed.namespace:
            logger.warning(
                f"Could not parse namespace for {package_name}, using fallback"
            )
            module_name = package_name.replace("-", ".")
        else:
            module_name = parsed.namespace

    imports = [module_name]

    # Construct the path to the actual module directory
    module_parts = module_name.split(".")
    module_dir = os.path.join(package_path, *module_parts)

    if not os.path.isdir(module_dir):
        logger.warning(
            f"Module directory not found for {package_name} at {module_dir}, using base import only"
        )
        return imports

    # Check for common submodules and only add if they exist
    submodules_to_check = ["aio", "models", "operations"]

    for submodule_name in submodules_to_check:
        submodule_path = os.path.join(module_dir, submodule_name)
        if os.path.isdir(submodule_path) and os.path.exists(
            os.path.join(submodule_path, "__init__.py")
        ):
            imports.append(f"{module_name}.{submodule_name}")

    # Check for aio.operations (nested submodule)
    aio_operations_path = os.path.join(module_dir, "aio", "operations")
    if os.path.isdir(aio_operations_path) and os.path.exists(
        os.path.join(aio_operations_path, "__init__.py")
    ):
        imports.append(f"{module_name}.aio.operations")

    return imports



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._operations import ApplicationDataOperations
from ._operations import AttachmentsOperations
from ._operations import BoundariesOperations
from ._operations import CropProductsOperations
from ._operations import CropsOperations
from ._operations import DeviceDataModelsOperations
from ._operations import DevicesOperations
from ._operations import FarmOperationsOperations
from ._operations import FarmsOperations
from ._operations import FieldsOperations
from ._operations import HarvestDataOperations
from ._operations import ImageProcessingOperations
from ._operations import InsightAttachmentsOperations
from ._operations import InsightsOperations
from ._operations import ManagementZonesOperations
from ._operations import ModelInferenceOperations
from ._operations import NutrientAnalysesOperations
from ._operations import OAuthProvidersOperations
from ._operations import OAuthTokensOperations
from ._operations import PartiesOperations
from ._operations import PlantingDataOperations
from ._operations import PlantTissueAnalysesOperations
from ._operations import PrescriptionMapsOperations
from ._operations import PrescriptionsOperations
from ._operations import ScenesOperations
from ._operations import SeasonalFieldsOperations
from ._operations import SeasonsOperations
from ._operations import SensorDataModelsOperations
from ._operations import SensorEventsOperations
from ._operations import SensorMappingsOperations
from ._operations import SensorPartnerIntegrationsOperations
from ._operations import SensorsOperations
from ._operations import SolutionInferenceOperations
from ._operations import TillageDataOperations
from ._operations import WeatherOperations
from ._operations import WeatherDataOperations
from ._operations import ZonesOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # type: ignore # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ApplicationDataOperations",
    "AttachmentsOperations",
    "BoundariesOperations",
    "CropProductsOperations",
    "CropsOperations",
    "DeviceDataModelsOperations",
    "DevicesOperations",
    "FarmOperationsOperations",
    "FarmsOperations",
    "FieldsOperations",
    "HarvestDataOperations",
    "ImageProcessingOperations",
    "InsightAttachmentsOperations",
    "InsightsOperations",
    "ManagementZonesOperations",
    "ModelInferenceOperations",
    "NutrientAnalysesOperations",
    "OAuthProvidersOperations",
    "OAuthTokensOperations",
    "PartiesOperations",
    "PlantingDataOperations",
    "PlantTissueAnalysesOperations",
    "PrescriptionMapsOperations",
    "PrescriptionsOperations",
    "ScenesOperations",
    "SeasonalFieldsOperations",
    "SeasonsOperations",
    "SensorDataModelsOperations",
    "SensorEventsOperations",
    "SensorMappingsOperations",
    "SensorPartnerIntegrationsOperations",
    "SensorsOperations",
    "SolutionInferenceOperations",
    "TillageDataOperations",
    "WeatherOperations",
    "WeatherDataOperations",
    "ZonesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.appcontainers import ContainerAppsAPIClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-appcontainers
# USAGE
    python java_components_patch_service_bind.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ContainerAppsAPIClient(
        credential=DefaultAzureCredential(),
        subscription_id="8efdecc5-919e-44eb-b179-915dca89ebf9",
    )

    response = client.java_components.begin_update(
        resource_group_name="examplerg",
        environment_name="myenvironment",
        name="myjavacomponent",
        java_component_envelope={
            "properties": {
                "componentType": "SpringBootAdmin",
                "configurations": [
                    {"propertyName": "spring.boot.admin.ui.enable-toasts", "value": "true"},
                    {"propertyName": "spring.boot.admin.monitor.status-interval", "value": "10000ms"},
                ],
                "scale": {"maxReplicas": 1, "minReplicas": 1},
                "serviceBinds": [
                    {
                        "name": "yellowcat",
                        "serviceId": "/subscriptions/8efdecc5-919e-44eb-b179-915dca89ebf9/resourceGroups/examplerg/providers/Microsoft.App/managedEnvironments/myenvironment/javaComponents/yellowcat",
                    }
                ],
            }
        },
    ).result()
    print(response)


# x-ms-original-file: specification/app/resource-manager/Microsoft.App/ContainerApps/stable/2025-07-01/examples/JavaComponents_Patch_ServiceBind.json
if __name__ == "__main__":
    main()



# coding=utf-8

from enum import Enum
from corehttp.utils import CaseInsensitiveEnumMeta


class Versions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The version of the API."""

    V1 = "v1"
    """The version v1."""
    V2 = "v2"
    """The version v2."""



# coding: utf-8
# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from devtools_testutils import AzureRecordedTestCase
from {{ namespace }}.aio import {{ client_name}}


class {{ test_prefix.capitalize() }}AsyncTest(AzureRecordedTestCase):
    def create_client(self, endpoint):
        credential = self.get_credential({{ client_name}}, is_async=True)
        return self.create_client_from_credential(
            {{ client_name}},
            credential=credential,
            endpoint=endpoint,
        )



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_chat.py

DESCRIPTION:
    Demonstrate a follow-up (contextual) question using `answer_context`.

USAGE:
    python sample_chat.py

Environment variables:
    1) AZURE_QUESTIONANSWERING_ENDPOINT
    2) AZURE_QUESTIONANSWERING_KEY
    3) AZURE_QUESTIONANSWERING_PROJECT
    4) AZURE_QUESTIONANSWERING_DEPLOYMENT (optional; defaults to 'production')
"""


from __future__ import annotations


def sample_chit_chat():
    # [START chit_chat]
    import os
    from azure.core.credentials import AzureKeyCredential
    from azure.ai.language.questionanswering import QuestionAnsweringClient
    from azure.ai.language.questionanswering.models import (
        AnswersOptions,
        ShortAnswerOptions,
        KnowledgeBaseAnswerContext,
    )

    endpoint = os.environ["AZURE_QUESTIONANSWERING_ENDPOINT"]
    key = os.environ["AZURE_QUESTIONANSWERING_KEY"]
    project = os.environ["AZURE_QUESTIONANSWERING_PROJECT"]
    deployment = os.environ.get("AZURE_QUESTIONANSWERING_DEPLOYMENT", "production")

    client = QuestionAnsweringClient(endpoint, AzureKeyCredential(key))
    with client:
        first_question = "How long should my Surface battery last?"

        first_options = AnswersOptions(
            question=first_question,
            top=3,
            confidence_threshold=0.2,
            include_unstructured_sources=True,
            short_answer_options=ShortAnswerOptions(enable=True, confidence_threshold=0.2, top=1),
        )
        first_output = client.get_answers(
            first_options,
            project_name=project,
            deployment_name=deployment,
        )
        best_candidate = next(
            (a for a in (first_output.answers or []) if a.confidence and a.confidence > 0.7),
            None,
        )
        if not best_candidate:
            print(f"No high-confidence answers for '{first_question}'")
            return

        print(f"Q: {first_question}")
        print(f"A: {best_candidate.answer}")

        if best_candidate.qna_id:
            followup_question = "How long does it take to charge a Surface?"
            # Use answer_context to provide previous QnA selection for disambiguation.
            followup_options = AnswersOptions(
                question=followup_question,
                top=3,
                confidence_threshold=0.2,
                answer_context=KnowledgeBaseAnswerContext(
                    previous_question=first_question,
                    previous_qna_id=best_candidate.qna_id,
                ),
                short_answer_options=ShortAnswerOptions(enable=True, confidence_threshold=0.2, top=1),
                include_unstructured_sources=True,
            )
            followup_output = client.get_answers(
                followup_options,
                project_name=project,
                deployment_name=deployment,
            )
            follow_best = next(
                (a for a in (followup_output.answers or []) if a.confidence and a.confidence > 0.2),
                None,
            )
            if follow_best:
                print(f"Q: {followup_question}")
                print(f"A: {follow_best.answer}")
            else:
                print(f"No answers for follow-up '{followup_question}'")
    # [END chit_chat]


if __name__ == "__main__":
    sample_chit_chat()



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import azure.ai.inference as sdk
import azure.ai.inference.aio as async_sdk
import functools
import io
import json
import logging
import re
import sys

from os import path
from pathlib import Path
from typing import List, Optional, Union, Dict, Any
from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader
from azure.core.credentials import AzureKeyCredential
from azure.core.rest import HttpResponse, AsyncHttpResponse

# Set to True to enable SDK logging
LOGGING_ENABLED = True

if LOGGING_ENABLED:
    # Create a logger for the 'azure' SDK
    # See https://docs.python.org/3/library/logging.html
    logger = logging.getLogger("azure")
    logger.setLevel(logging.DEBUG)  # INFO or DEBUG

    # Configure a console output
    handler = logging.StreamHandler(stream=sys.stdout)
    logger.addHandler(handler)

#
# Define these environment variables. They should point to a Mistral Large model
# hosted on MaaS, or any other MaaS model that suppots chat completions with tools.
# AZURE_AI_CHAT_ENDPOINT=https://<endpoint-name>.<azure-region>.models.ai.azure.com
# AZURE_AI_CHAT_KEY=<api-key>
#
ServicePreparerChatCompletions = functools.partial(
    EnvironmentVariableLoader,
    "azure_ai_chat",
    azure_ai_chat_endpoint="https://your-deployment-name.eastus2.models.ai.azure.com",
    azure_ai_chat_key="00000000000000000000000000000000",
    azure_ai_chat_model="mistral-large-2411",
)

#
# Define these environment variables. They should point to any GPT model that
# accepts image input in chat completions (e.g. GPT-4o model).
# hosted on Azure OpenAI (AOAI) endpoint.
# TODO: When we have a MaaS model that supports chat completions with image input,
# use that instead.
# AZURE_OPENAI_CHAT_ENDPOINT=https://<endpont-name>.openai.azure.com/openai/deployments/gpt-4o
# AZURE_OPENAI_CHAT_KEY=<api-key>
#
ServicePreparerAOAIChatCompletions = functools.partial(
    EnvironmentVariableLoader,
    "azure_openai_chat",
    azure_openai_chat_endpoint="https://your-deployment-name.openai.azure.com/openai/deployments/gpt-4o-deployment",
    azure_openai_chat_key="00000000000000000000000000000000",
    azure_openai_chat_api_version="yyyy-mm-dd-preview",
    azure_openai_chat_audio_endpoint="https://your-deployment-name.openai.azure.com/openai/deployments/gpt-4o-audio-preview",
    azure_openai_chat_audio_key="00000000000000000000000000000000",
    azure_openai_chat_audio_api_version="yyyy-mm-dd-preview",
)

#
# Define these environment variables for text embeddings. They should point to a Cohere model
# hosted on MaaS, or any other MaaS model that text embeddings.
# AZURE_AI_EMBEDDINGS_ENDPOINT=https://<endpoint-name>.<azure-region>.models.ai.azure.com
# AZURE_AI_EMBEDDINGS_KEY=<pi-key>
#
ServicePreparerEmbeddings = functools.partial(
    EnvironmentVariableLoader,
    "azure_ai_embeddings",
    azure_ai_embeddings_endpoint="https://your-deployment-name.eastus2.models.ai.azure.com",
    azure_ai_embeddings_key="00000000000000000000000000000000",
)

#
# Define these environment variables for image embeddings. They should point to a Cohere model
# hosted on MaaS, or any other MaaS model that text embeddings.
# AZURE_AI_IMAGE_EMBEDDINGS_ENDPOINT=https://<endpoint-name>.<azure-region>.models.ai.azure.com
# AZURE_AI_IMAGE_EMBEDDINGS_KEY=<api-key>
#
ServicePreparerImageEmbeddings = functools.partial(
    EnvironmentVariableLoader,
    "azure_ai_image_embeddings",
    azure_ai_image_embeddings_endpoint="https://your-deployment-name.eastus2.models.ai.azure.com",
    azure_ai_image_embeddings_key="00000000000000000000000000000000",
)


# The test class name needs to start with "Test" to get collected by pytest
class ModelClientTestBase(AzureRecordedTestCase):

    # Set to True to print out all results to the console
    PRINT_RESULT = True

    # Regular expression describing the pattern of a result ID returned from MaaS/MaaP endpoint. Format allowed are:
    # "183b56eb-8512-484d-be50-5d8df82301a2", "26ef25aa45424781865a2d38a4484274" and "Sanitized" (when running tests
    # from recordings)
    REGEX_RESULT_ID = re.compile(
        r"^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$|^[0-9a-fA-F]{32}$|^Sanitized$"
    )

    # Regular expression describing the pattern of a result ID returned from AOAI endpoint.
    # For example: "chatcmpl-9jscXwejvOMnGrxRfACmNrCCdiwWb" or "Sanitized" (when runing tests from recordings) # cspell:disable-line
    REGEX_AOAI_RESULT_ID = re.compile(r"^chatcmpl-[0-9a-zA-Z]{29}$|^Sanitized$")  # cspell:disable-line

    # Regular expression describing the pattern of a base64 string
    REGEX_BASE64_STRING = re.compile(r"^[A-Za-z0-9+/]+={0,3}$")

    # A couple of tool definitions to use in the tests
    TOOL1 = sdk.models.ChatCompletionsToolDefinition(
        function=sdk.models.FunctionDefinition(
            name="my-first-function-name",
            description="My first function description",
            parameters={
                "type": "object",
                "properties": {
                    "first_argument": {
                        "type": "string",
                        "description": "First argument description",
                    },
                    "second_argument": {
                        "type": "string",
                        "description": "Second argument description",
                    },
                },
                "required": ["first_argument", "second_argument"],
            },
        )
    )

    TOOL2 = sdk.models.ChatCompletionsToolDefinition(
        function=sdk.models.FunctionDefinition(
            name="my-second-function-name",
            description="My second function description",
            parameters={
                "type": "object",
                "properties": {
                    "first_argument": {
                        "type": "int",
                        "description": "First argument description",
                    },
                },
                "required": ["first_argument"],
            },
        )
    )

    # Expected JSON request payload in regression tests. These are common to
    # sync and async tests, therefore they are defined here.
    CHAT_COMPLETIONS_JSON_REQUEST_PAYLOAD = '{"messages": [{"role": "system", "content": "system prompt"}, {"role": "user", "content": "user prompt 1"}, {"role": "assistant", "tool_calls": [{"function": {"name": "my-first-function-name", "arguments": {"first_argument": "value1", "second_argument": "value2"}}, "id": "some-id", "type": "function"}, {"function": {"name": "my-second-function-name", "arguments": {"first_argument": "value1"}}, "id": "some-other-id", "type": "function"}]}, {"role": "tool", "tool_call_id": "some id", "content": "function response"}, {"role": "assistant", "content": "assistant prompt"}, {"role": "user", "content": [{"type": "text", "text": "user prompt 2"}, {"type": "image_url", "image_url": {"url": "https://does.not.exit/image.png", "detail": "high"}}]}], "stream": true, "frequency_penalty": 0.123, "max_tokens": 321, "model": "some-model-id", "presence_penalty": 4.567, "response_format": {"type": "json_object"}, "seed": 654, "stop": ["stop1", "stop2"], "temperature": 8.976, "tool_choice": "auto", "tools": [{"function": {"name": "my-first-function-name", "description": "My first function description", "parameters": {"type": "object", "properties": {"first_argument": {"type": "string", "description": "First argument description"}, "second_argument": {"type": "string", "description": "Second argument description"}}, "required": ["first_argument", "second_argument"]}}, "type": "function"}, {"function": {"name": "my-second-function-name", "description": "My second function description", "parameters": {"type": "object", "properties": {"first_argument": {"type": "int", "description": "First argument description"}}, "required": ["first_argument"]}}, "type": "function"}], "top_p": 9.876, "key1": 1, "key2": true, "key3": "Some value", "key4": [1, 2, 3], "key5": {"key6": 2, "key7": false, "key8": "Some other value", "key9": [4, 5, 6, 7]}}'

    EMBEDDINGDS_JSON_REQUEST_PAYLOAD = '{"input": ["first phrase", "second phrase", "third phrase"], "dimensions": 2048, "encoding_format": "ubinary", "input_type": "query", "model": "some-model-id", "key1": 1, "key2": true, "key3": "Some value", "key4": [1, 2, 3], "key5": {"key6": 2, "key7": false, "key8": "Some other value", "key9": [4, 5, 6, 7]}}'

    IMAGE_EMBEDDINGDS_JSON_REQUEST_PAYLOAD = '{"input": [{"image": "data:image/png;base64, ", "text": "some text"}], "dimensions": 2048, "encoding_format": "ubinary", "input_type": "query", "model": "some-model-id", "key1": 1, "key2": true, "key3": "Some value", "key4": [1, 2, 3], "key5": {"key6": 2, "key7": false, "key8": "Some other value", "key9": [4, 5, 6, 7]}}'

    OUTPUT_FORMAT_JSON_SCHEMA: Dict[str, Any] = {
        "type": "object",
        "properties": {
            "distances": {
                "type": "array",
                "items": {
                    "type": "object",
                    "properties": {
                        "location1": {
                            "type": "string",
                            "description": "The name of the first location",
                        },
                        "location2": {
                            "type": "string",
                            "description": "The name of the second location",
                        },
                        "distance": {
                            "type": "integer",
                            "description": "The distance between the two locations in miles",
                        },
                    },
                    "required": ["location1", "location2", "distance"],
                    "additionalProperties": False,
                },
            },
        },
        "required": ["distances"],
        "additionalProperties": False,
    }

    # **********************************************************************************
    #
    #              HELPER METHODS TO LOAD AUTH CREDENTIALS FOR ALL CLIENTS
    #
    # **********************************************************************************

    # Method to load chat completions api-key credentials from environment variables
    def _load_chat_credentials_api_key(self, *, bad_key: bool, **kwargs):
        endpoint = kwargs.pop("azure_ai_chat_endpoint")
        key = "00000000000000000000000000000000" if bad_key else kwargs.pop("azure_ai_chat_key")
        credential = AzureKeyCredential(key)
        return endpoint, credential

    # Method to load chat completions Entra ID credentials from environment variables
    def _load_chat_credentials_entra_id(self, *, is_async: bool = False, **kwargs):
        endpoint = kwargs.pop("azure_ai_chat_endpoint")
        credential = self.get_credential(sdk.ChatCompletionsClient, is_async=is_async)
        return endpoint, credential

    # Method to load chat completions credentials when using OpenAI models.
    # See the "Data plane - inference" row in the table here for latest AOAI api-version:
    # https://aka.ms/azsdk/azure-ai-inference/azure-openai-api-versions
    def _load_aoai_chat_credentials(self, *, key_auth: bool, bad_key: bool, is_async: bool = False, **kwargs):
        endpoint = kwargs.pop("azure_openai_chat_endpoint")
        api_version = kwargs.pop("azure_openai_chat_api_version")
        if key_auth:
            key = "00000000000000000000000000000000" if bad_key else kwargs.pop("azure_openai_chat_key")
            # We no longer need to set "api-key" header, since the SDK was updated to set this header
            # (both "api-key" header and "Authorization": "Bearer ..." headers are now used for api key auth).
            # headers = {"api-key": key}
            credential = AzureKeyCredential(key)
            credential_scopes: list[str] = []
        else:
            credential = self.get_credential(sdk.ChatCompletionsClient, is_async=is_async)
            credential_scopes: list[str] = ["https://cognitiveservices.azure.com/.default"]
            # headers = {}
        return endpoint, credential, credential_scopes, api_version  # , headers

    def _load_aoai_audio_chat_credentials(self, *, key_auth: bool, bad_key: bool, is_async: bool = False, **kwargs):
        endpoint = kwargs.pop("azure_openai_chat_audio_endpoint")
        api_version = kwargs.pop("azure_openai_chat_audio_api_version")
        if key_auth:
            key = "00000000000000000000000000000000" if bad_key else kwargs.pop("azure_openai_chat_audio_key")
            # We no longer need to set "api-key" header, since the SDK was updated to set this header
            # (both "api-key" header and "Authorization": "Bearer ..." headers are now used for api key auth).
            # headers = {"api-key": key}
            credential = AzureKeyCredential(key)
            credential_scopes: list[str] = []
        else:
            credential = self.get_credential(sdk.ChatCompletionsClient, is_async=is_async)
            credential_scopes: list[str] = ["https://cognitiveservices.azure.com/.default"]
            # headers = {}
        return endpoint, credential, credential_scopes, api_version  # , headers

    def _load_embeddings_credentials_api_key(self, *, bad_key: bool, **kwargs):
        endpoint = kwargs.pop("azure_ai_embeddings_endpoint")
        key = "00000000000000000000000000000000" if bad_key else kwargs.pop("azure_ai_embeddings_key")
        credential = AzureKeyCredential(key)
        return endpoint, credential

    def _load_embeddings_credentials_entra_id(self, is_async: bool = False, **kwargs):
        endpoint = kwargs.pop("azure_ai_embeddings_endpoint")
        credential = self.get_credential(sdk.EmbeddingsClient, is_async=is_async)
        return endpoint, credential

    def _load_image_embeddings_credentials_key_auth(self, *, bad_key: bool, **kwargs):
        endpoint = kwargs.pop("azure_ai_image_embeddings_endpoint")
        key = "00000000000000000000000000000000" if bad_key else kwargs.pop("azure_ai_image_embeddings_key")
        credential = AzureKeyCredential(key)
        return endpoint, credential

    def _load_image_embeddings_credentials_entra_id(self, is_async: bool = False, **kwargs):
        endpoint = kwargs.pop("azure_ai_image_embeddings_endpoint")
        credential = self.get_credential(sdk.ImageEmbeddingsClient, is_async=is_async)
        return endpoint, credential

    # **********************************************************************************
    #
    #     HELPER METHODS TO CREATE CLIENTS USING THE SDK's load_client() FUNCTION
    #
    # **********************************************************************************

    # Methods to create sync and async clients using Load_client() function
    async def _load_async_chat_client(self, *, bad_key: bool = False, **kwargs) -> async_sdk.ChatCompletionsClient:
        endpoint, credential = self._load_chat_credentials_api_key(bad_key=bad_key, **kwargs)
        return await async_sdk.load_client(endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED)

    def _load_chat_client(self, *, bad_key: bool = False, **kwargs) -> sdk.ChatCompletionsClient:
        endpoint, credential = self._load_chat_credentials_api_key(bad_key=bad_key, **kwargs)
        return sdk.load_client(endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED)

    def _load_chat_client_on_aoai_endpoint(self, *, bad_key: bool = False, **kwargs) -> sdk.ChatCompletionsClient:
        endpoint, credential, credential_scopes, api_version = self._load_aoai_chat_credentials(
            key_auth=True, bad_key=False, **kwargs
        )
        return sdk.load_client(
            endpoint=endpoint,
            credential=credential,
            credential_scopes=credential_scopes,
            api_version=api_version,
            logging_enable=LOGGING_ENABLED,
        )

    async def _load_async_embeddings_client(self, *, bad_key: bool = False, **kwargs) -> async_sdk.EmbeddingsClient:
        endpoint, credential = self._load_embeddings_credentials_api_key(bad_key=bad_key, **kwargs)
        return await async_sdk.load_client(endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED)

    def _load_embeddings_client(self, *, bad_key: bool = False, **kwargs) -> sdk.EmbeddingsClient:
        endpoint, credential = self._load_embeddings_credentials_api_key(bad_key=bad_key, **kwargs)
        return sdk.load_client(endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED)

    async def _load_async_image_embeddings_client(
        self, *, bad_key: bool = False, **kwargs
    ) -> async_sdk.ImageEmbeddingsClient:
        endpoint, credential = self._load_image_embeddings_credentials_key_auth(bad_key=bad_key, **kwargs)
        return await async_sdk.load_client(endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED)

    def _load_image_embeddings_client(self, *, bad_key: bool = False, **kwargs) -> sdk.ImageEmbeddingsClient:
        endpoint, credential = self._load_image_embeddings_credentials_key_auth(bad_key=bad_key, **kwargs)
        return sdk.load_client(endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED)

    # **********************************************************************************
    #
    #                  HELPER METHODS TO DIRECTLY CREATE CLIENTS
    #
    # **********************************************************************************

    def _create_chat_client(
        self, *, bad_key: bool = False, key_auth: bool = True, **kwargs
    ) -> sdk.ChatCompletionsClient:
        if key_auth:
            endpoint, credential = self._load_chat_credentials_api_key(bad_key=bad_key, **kwargs)
        else:
            endpoint, credential = self._load_chat_credentials_entra_id(**kwargs)
        return sdk.ChatCompletionsClient(
            endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED, **kwargs
        )

    # Methos to create the different sync and async clients directly
    def _create_async_chat_client(
        self, *, bad_key: bool = False, key_auth: bool = True, **kwargs
    ) -> async_sdk.ChatCompletionsClient:
        if key_auth:
            endpoint, credential = self._load_chat_credentials_api_key(bad_key=bad_key, **kwargs)
        else:
            endpoint, credential = self._load_chat_credentials_entra_id(is_async=True, **kwargs)
        return async_sdk.ChatCompletionsClient(
            endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED, **kwargs
        )

    def _create_aoai_chat_client(
        self, *, key_auth: bool = True, bad_key: bool = False, **kwargs
    ) -> sdk.ChatCompletionsClient:
        endpoint, credential, credential_scopes, api_version = self._load_aoai_chat_credentials(
            key_auth=key_auth, bad_key=bad_key, **kwargs
        )
        return sdk.ChatCompletionsClient(
            endpoint=endpoint,
            credential=credential,
            credential_scopes=credential_scopes,
            api_version=api_version,
            logging_enable=LOGGING_ENABLED,
        )

    def _create_async_aoai_chat_client(
        self, *, key_auth: bool = True, bad_key: bool = False, **kwargs
    ) -> async_sdk.ChatCompletionsClient:
        endpoint, credential, credential_scopes, api_version = self._load_aoai_chat_credentials(
            key_auth=True, bad_key=bad_key, is_async=True, **kwargs
        )
        return async_sdk.ChatCompletionsClient(
            endpoint=endpoint,
            credential=credential,
            credential_scopes=credential_scopes,
            api_version=api_version,
            logging_enable=LOGGING_ENABLED,
        )

    def _create_aoai_audio_chat_client(
        self, *, key_auth: bool = True, bad_key: bool = False, **kwargs
    ) -> sdk.ChatCompletionsClient:
        endpoint, credential, credential_scopes, api_version = self._load_aoai_audio_chat_credentials(
            key_auth=key_auth, bad_key=bad_key, **kwargs
        )
        return sdk.ChatCompletionsClient(
            endpoint=endpoint,
            credential=credential,
            credential_scopes=credential_scopes,
            api_version=api_version,
            logging_enable=LOGGING_ENABLED,
        )

    def _create_async_aoai_audio_chat_client(
        self, *, key_auth: bool = True, bad_key: bool = False, **kwargs
    ) -> async_sdk.ChatCompletionsClient:
        endpoint, credential, credential_scopes, api_version = self._load_aoai_audio_chat_credentials(
            key_auth=True, bad_key=bad_key, is_async=True, **kwargs
        )
        return async_sdk.ChatCompletionsClient(
            endpoint=endpoint,
            credential=credential,
            credential_scopes=credential_scopes,
            api_version=api_version,
            logging_enable=LOGGING_ENABLED,
        )

    def _create_async_embeddings_client(
        self, *, bad_key: bool = False, key_auth: bool = True, **kwargs
    ) -> async_sdk.EmbeddingsClient:
        if key_auth:
            endpoint, credential = self._load_embeddings_credentials_api_key(bad_key=bad_key, **kwargs)
        else:
            endpoint, credential = self._load_embeddings_credentials_entra_id(is_async=True, **kwargs)
        return async_sdk.EmbeddingsClient(
            endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED, **kwargs
        )

    def _create_embeddings_client(
        self, *, bad_key: bool = False, key_auth: bool = True, **kwargs
    ) -> sdk.EmbeddingsClient:
        if key_auth:
            endpoint, credential = self._load_embeddings_credentials_api_key(bad_key=bad_key, **kwargs)
        else:
            endpoint, credential = self._load_embeddings_credentials_entra_id(**kwargs)
        return sdk.EmbeddingsClient(endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED, **kwargs)

    def _create_image_embeddings_client(
        self, *, bad_key: bool = False, key_auth: bool = True, **kwargs
    ) -> sdk.ImageEmbeddingsClient:
        if key_auth:
            endpoint, credential = self._load_image_embeddings_credentials_key_auth(bad_key=bad_key, **kwargs)
        else:
            endpoint, credential = self._load_image_embeddings_credentials_entra_id(**kwargs)
        return sdk.ImageEmbeddingsClient(
            endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED, **kwargs
        )

    def _create_async_image_embeddings_client(
        self, *, bad_key: bool = False, key_auth: bool = True, **kwargs
    ) -> async_sdk.ImageEmbeddingsClient:
        if key_auth:
            endpoint, credential = self._load_image_embeddings_credentials_key_auth(bad_key=bad_key, **kwargs)
        else:
            endpoint, credential = self._load_image_embeddings_credentials_entra_id(is_async=True, **kwargs)
        return async_sdk.ImageEmbeddingsClient(
            endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED, **kwargs
        )

    def _create_embeddings_client_with_chat_completions_credentials(self, **kwargs) -> sdk.EmbeddingsClient:
        endpoint = kwargs.pop("azure_ai_chat_endpoint")
        key = kwargs.pop("azure_ai_chat_key")
        credential = AzureKeyCredential(key)
        return sdk.EmbeddingsClient(endpoint=endpoint, credential=credential, logging_enable=LOGGING_ENABLED)

    # **********************************************************************************
    #
    #             HELPER METHODS TO VALIDATE TEST RESULTS
    #
    # **********************************************************************************

    def _validate_embeddings_json_request_payload(self) -> None:
        headers = self.pipeline_request.http_request.headers
        print(f"Actual HTTP request headers: {self.pipeline_request.http_request.headers}")
        print(f"Actual JSON request payload: {self.pipeline_request.http_request.data}")
        assert headers["Content-Type"] == "application/json"
        assert headers["Content-Length"] == "311"
        assert headers["extra-parameters"] == "pass-through"
        assert headers["Accept"] == "application/json"
        assert headers["some_header"] == "some_header_value"
        assert "MyAppId azsdk-python-ai-inference/" in headers["User-Agent"]
        assert " Python/" in headers["User-Agent"]
        assert headers["Authorization"] == "Bearer key-value"
        assert headers["api-key"] == "key-value"
        assert self.pipeline_request.http_request.data == self.EMBEDDINGDS_JSON_REQUEST_PAYLOAD

    def _validate_image_embeddings_json_request_payload(self) -> None:
        headers = self.pipeline_request.http_request.headers
        print(f"Actual HTTP request headers: {self.pipeline_request.http_request.headers}")
        print(f"Actual JSON request payload: {self.pipeline_request.http_request.data}")
        assert headers["Content-Type"] == "application/json"
        assert headers["Content-Length"] == "10364"
        assert headers["extra-parameters"] == "pass-through"
        assert headers["Accept"] == "application/json"
        assert headers["some_header"] == "some_header_value"
        assert "MyAppId azsdk-python-ai-inference/" in headers["User-Agent"]
        assert " Python/" in headers["User-Agent"]
        assert headers["Authorization"] == "Bearer key-value"
        assert headers["api-key"] == "key-value"
        assert self.pipeline_request.http_request.data == self.IMAGE_EMBEDDINGDS_JSON_REQUEST_PAYLOAD

    def _validate_chat_completions_json_request_payload(self) -> None:
        print(f"Actual HTTP request headers: {self.pipeline_request.http_request.headers}")
        print(f"Actual JSON request payload: {self.pipeline_request.http_request.data}")
        headers = self.pipeline_request.http_request.headers
        assert headers["Content-Type"] == "application/json"
        assert headers["Content-Length"] == "1840"
        assert headers["extra-parameters"] == "pass-through"
        assert headers["Accept"] == "application/json"
        assert headers["some_header"] == "some_header_value"
        assert "MyAppId azsdk-python-ai-inference/" in headers["User-Agent"]
        assert " Python/" in headers["User-Agent"]
        assert headers["Authorization"] == "Bearer key-value"
        assert headers["api-key"] == "key-value"
        assert self.pipeline_request.http_request.data == self.CHAT_COMPLETIONS_JSON_REQUEST_PAYLOAD

    @staticmethod
    def _validate_model_info_result(
        model_info: sdk.models.ModelInfo, expected_model_type: Union[str, sdk.models.ModelType]
    ):
        assert model_info.model_name is not None
        assert len(model_info.model_name) > 0
        assert model_info.model_provider_name is not None
        assert len(model_info.model_provider_name) > 0
        assert model_info.model_type is not None
        assert model_info.model_type == expected_model_type

    @staticmethod
    def _validate_model_extras(body: str, headers: Dict[str, str]):
        assert headers is not None
        assert headers["extra-parameters"] == "pass-through"
        assert body is not None
        try:
            body_json = json.loads(body)
        except json.JSONDecodeError:
            print("Invalid JSON format")
        assert body_json["n"] == 1

    @staticmethod
    def _validate_chat_completions_result(
        response: sdk.models.ChatCompletions,
        contains: List[str],
        *,
        is_aoai: Optional[bool] = False,
        is_json: Optional[bool] = False,
    ):
        assert any(item in response.choices[0].message.content for item in contains)
        assert response.choices[0].message.role == sdk.models.ChatRole.ASSISTANT
        assert response.choices[0].finish_reason == sdk.models.CompletionsFinishReason.STOPPED
        assert response.choices[0].index == 0
        if is_aoai:
            assert bool(ModelClientTestBase.REGEX_AOAI_RESULT_ID.match(response.id))
        else:
            assert bool(ModelClientTestBase.REGEX_RESULT_ID.match(response.id))
        assert response.created is not None
        assert response.created != ""
        assert response.model is not None
        assert response.model != ""
        assert response.usage.prompt_tokens > 0
        assert response.usage.completion_tokens > 0
        assert response.usage.total_tokens == response.usage.prompt_tokens + response.usage.completion_tokens
        if is_json:
            # Validate legal JSON format by parsing it
            json_data = json.loads(response.choices[0].message.content)

    @staticmethod
    def _validate_chat_completions_tool_result(response: sdk.models.ChatCompletions):
        assert response.choices[0].message.content == None or response.choices[0].message.content == ""
        assert response.choices[0].message.role == sdk.models.ChatRole.ASSISTANT
        assert response.choices[0].finish_reason == sdk.models.CompletionsFinishReason.TOOL_CALLS
        assert response.choices[0].index == 0
        function_args = json.loads(response.choices[0].message.tool_calls[0].function.arguments.replace("'", '"'))
        print(function_args)
        assert function_args["city"].lower() == "seattle"
        assert function_args["days"] == "2"
        assert bool(ModelClientTestBase.REGEX_RESULT_ID.match(response.id))
        assert response.created is not None
        assert response.created != ""
        assert response.model is not None
        # assert response.model != ""
        assert response.usage.prompt_tokens > 0
        assert response.usage.completion_tokens > 0
        assert response.usage.total_tokens == response.usage.prompt_tokens + response.usage.completion_tokens

    @staticmethod
    def _validate_chat_completions_update(update: sdk.models.StreamingChatCompletionsUpdate, first: bool) -> str:
        if first:
            # Why is 'content','created' and 'object' missing in the first update?
            assert update.choices[0].delta.role == sdk.models.ChatRole.ASSISTANT
        else:
            assert update.choices[0].delta.role == None
            assert update.choices[0].delta.content != None
            assert update.created is not None
            assert update.created != ""
        assert update.choices[0].delta.tool_calls == None
        assert update.choices[0].index == 0
        assert update.id is not None
        assert bool(ModelClientTestBase.REGEX_RESULT_ID.match(update.id))
        assert update.model is not None
        assert update.model != ""
        if update.choices[0].delta.content != None:
            return update.choices[0].delta.content
        else:
            return ""

    @staticmethod
    def _validate_chat_completions_streaming_result(response: sdk.models.StreamingChatCompletions):
        count = 0
        content = ""
        for update in response:
            content += ModelClientTestBase._validate_chat_completions_update(update, count == 0)
            count += 1
        assert count > 2
        assert len(content) > 100  # Some arbitrary number
        # The last update should have a finish reason and usage
        assert update.choices[0].finish_reason == sdk.models.CompletionsFinishReason.STOPPED
        assert update.usage.prompt_tokens > 0
        assert update.usage.completion_tokens > 0
        assert update.usage.total_tokens == update.usage.prompt_tokens + update.usage.completion_tokens
        if ModelClientTestBase.PRINT_RESULT:
            print(content)

    @staticmethod
    async def _validate_async_chat_completions_streaming_result(response: sdk.models.AsyncStreamingChatCompletions):
        count = 0
        content = ""
        async for update in response:
            content += ModelClientTestBase._validate_chat_completions_update(update, count == 0)
            count += 1
        assert count > 2
        assert len(content) > 100  # Some arbitrary number
        # The last update should have a finish reason and usage
        assert update.choices[0].finish_reason == sdk.models.CompletionsFinishReason.STOPPED
        assert update.usage.prompt_tokens > 0
        assert update.usage.completion_tokens > 0
        assert update.usage.total_tokens == update.usage.prompt_tokens + update.usage.completion_tokens
        if ModelClientTestBase.PRINT_RESULT:
            print(content)

    @staticmethod
    def _validate_embeddings_result(
        response: sdk.models.EmbeddingsResult,
        encoding_format: sdk.models.EmbeddingEncodingFormat = sdk.models.EmbeddingEncodingFormat.FLOAT,
    ):
        assert response is not None
        assert response.data is not None
        assert len(response.data) == 3
        for i in [0, 1, 2]:
            assert response.data[i] is not None
            assert response.data[i].index == i
            if encoding_format == sdk.models.EmbeddingEncodingFormat.FLOAT:
                assert isinstance(response.data[i].embedding, List)
                assert len(response.data[i].embedding) > 0
                assert response.data[i].embedding[0] != 0.0
                assert response.data[i].embedding[-1] != 0.0
            elif encoding_format == sdk.models.EmbeddingEncodingFormat.BASE64:
                assert isinstance(response.data[i].embedding, str)
                assert len(response.data[i].embedding) > 0
                assert bool(ModelClientTestBase.REGEX_BASE64_STRING.match(response.data[i].embedding))  # type: ignore[arg-type]
            else:
                raise ValueError(f"Unsupported encoding format: {encoding_format}")
        assert bool(ModelClientTestBase.REGEX_RESULT_ID.match(response.id))
        # assert len(response.model) > 0  # At the time of writing this test, this JSON field existed but was empty
        assert response.usage.prompt_tokens > 0
        assert response.usage.total_tokens == response.usage.prompt_tokens

    @staticmethod
    def _validate_image_embeddings_result(
        response: sdk.models.EmbeddingsResult,
        encoding_format: sdk.models.EmbeddingEncodingFormat = sdk.models.EmbeddingEncodingFormat.FLOAT,
    ):
        assert response is not None
        assert response.data is not None
        assert len(response.data) == 1
        for i in [0]:
            assert response.data[i] is not None
            assert response.data[i].index == i
            if encoding_format == sdk.models.EmbeddingEncodingFormat.FLOAT:
                assert isinstance(response.data[i].embedding, List)
                assert len(response.data[i].embedding) > 0
                assert response.data[i].embedding[0] != 0.0
                assert response.data[i].embedding[-1] != 0.0
            elif encoding_format == sdk.models.EmbeddingEncodingFormat.BASE64:
                assert isinstance(response.data[i].embedding, str)
                assert len(response.data[i].embedding) > 0
                assert bool(ModelClientTestBase.REGEX_BASE64_STRING.match(response.data[i].embedding))  # type: ignore[arg-type]
            else:
                raise ValueError(f"Unsupported encoding format: {encoding_format}")
        assert bool(ModelClientTestBase.REGEX_RESULT_ID.match(response.id))
        # assert len(response.model) > 0  # At the time of writing this test, this JSON field existed but was empty
        assert response.usage.prompt_tokens > 0
        assert response.usage.total_tokens == response.usage.prompt_tokens

    # **********************************************************************************
    #
    #                   HELPER METHODS TO PRINT RESULTS
    #
    # **********************************************************************************

    @staticmethod
    def _print_model_info_result(model_info: sdk.models.ModelInfo):
        if ModelClientTestBase.PRINT_RESULT:
            print(" Model info:")
            print("\tmodel_name: {}".format(model_info.model_name))
            print("\tmodel_type: {}".format(model_info.model_type))
            print("\tmodel_provider_name: {}".format(model_info.model_provider_name))

    @staticmethod
    def _print_chat_completions_result(response: sdk.models.ChatCompletions):
        if ModelClientTestBase.PRINT_RESULT:
            print(" Chat Completions response:")
            for choice in response.choices:
                print(f"\tchoices[0].message.content: {choice.message.content}")
                print(f"\tchoices[0].message.tool_calls: {choice.message.tool_calls}")
                print("\tchoices[0].message.role: {}".format(choice.message.role))
                print("\tchoices[0].finish_reason: {}".format(choice.finish_reason))
                print("\tchoices[0].index: {}".format(choice.index))
            print("\tid: {}".format(response.id))
            print("\tcreated: {}".format(response.created))
            print("\tmodel: {}".format(response.model))
            print("\tusage.prompt_tokens: {}".format(response.usage.prompt_tokens))
            print("\tusage.completion_tokens: {}".format(response.usage.completion_tokens))
            print("\tusage.total_tokens: {}".format(response.usage.total_tokens))

    @staticmethod
    def _print_embeddings_result(
        response: sdk.models.EmbeddingsResult,
        encoding_format: sdk.models.EmbeddingEncodingFormat = sdk.models.EmbeddingEncodingFormat.FLOAT,
    ):
        if ModelClientTestBase.PRINT_RESULT:
            print("Embeddings response:")
            for item in response.data:
                if encoding_format == sdk.models.EmbeddingEncodingFormat.FLOAT:
                    length = len(item.embedding)
                    print(
                        f"data[{item.index}] (vector length={length}): "
                        f"[{item.embedding[0]}, {item.embedding[1]}, "
                        f"..., {item.embedding[length-2]}, {item.embedding[length-1]}]"
                    )
                elif encoding_format == sdk.models.EmbeddingEncodingFormat.BASE64:
                    print(
                        f"data[{item.index}] encoded (string length={len(item.embedding)}): "
                        f'"{item.embedding[:32]}...{item.embedding[-32:]}"'
                    )
                else:
                    raise ValueError(f"Unsupported encoding format: {encoding_format}")
            print(f"\tid: {response.id}")
            print(f"\tmodel: {response.model}")
            print(f"\tusage.prompt_tokens: {response.usage.prompt_tokens}")
            print(f"\tusage.total_tokens: {response.usage.total_tokens}")

    # **********************************************************************************
    #
    #                         OTHER HELPER METHODS
    #
    # **********************************************************************************

    def request_callback(self, pipeline_request) -> None:
        self.pipeline_request = pipeline_request

    @staticmethod
    def _get_image_embeddings_input(with_text: Optional[bool] = True) -> sdk.models.ImageEmbeddingInput:
        local_folder = path.dirname(path.abspath(__file__))
        image_file = path.join(local_folder, "test_image1.png")
        if with_text:
            return sdk.models.ImageEmbeddingInput.load(
                image_file=image_file,
                image_format="png",
                text="some text",
            )
        else:
            return sdk.models.ImageEmbeddingInput.load(
                image_file=image_file,
                image_format="png",
            )

    @staticmethod
    def _read_text_file(file_name: str) -> io.BytesIO:
        """
        Reads a text file and returns a BytesIO object with the file content in UTF-8 encoding.
        The file is expected to be in the same directory as this Python script.
        """
        with Path(__file__).with_name(file_name).open("r") as f:
            return io.BytesIO(f.read().encode("utf-8"))


# **********************************************************************************
#
#                         OTHER HELPER CLASSES
#
# **********************************************************************************


class HttpResponseForUnitTests(HttpResponse):

    def __init__(self, response_bytes: List[bytes]):
        self._response_bytes = response_bytes

    # Required to support context management
    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    # Implementation of HttpResponse abstract base class methods.
    # Only the first one requires an implementation for this test to work.
    def iter_bytes(self):
        for response_byte in self._response_bytes:
            yield response_byte

    def close(self):
        pass

    def iter_raw(self):
        pass

    def json(self):
        pass

    def raise_for_status(self):
        pass

    def text(self):
        pass

    def read(self):
        pass

    # Implementation of HttpResponse abstract base class properties
    # None of these are used by test code.
    @property
    def content(self):
        pass

    @property
    def content_type(self):
        pass

    @property
    def encoding(self):
        pass

    @property
    def headers(self):
        pass

    @property
    def is_closed(self):
        pass

    @property
    def is_stream_consumed(self):
        pass

    @property
    def reason(self):
        pass

    @property
    def request(self):
        pass

    @property
    def status_code(self):
        pass

    @property
    def url(self):
        pass


class AsyncHttpResponseForUnitTests(AsyncHttpResponse):

    def __init__(self, response_bytes: List[bytes]):
        self._response_bytes = response_bytes

    # Required to support context management
    def __aenter__(self):
        return self

    def __aexit__(self, exc_type, exc_val, exc_tb):
        pass

    # Implementation of HttpResponse abstract base class methods.
    # Only the first one requires an implementation for this test to work.
    async def iter_bytes(self):
        for response_byte in self._response_bytes:
            yield response_byte

    async def close(self):
        pass

    async def iter_raw(self):
        pass

    async def json(self):
        pass

    async def raise_for_status(self):
        pass

    async def text(self):
        pass

    async def read(self):
        pass

    # Implementation of HttpResponse abstract base class properties
    # None of these are used by test code.
    @property
    def content(self):
        pass

    @property
    def content_type(self):
        pass

    @property
    def encoding(self):
        pass

    @property
    def headers(self):
        pass

    @property
    def is_closed(self):
        pass

    @property
    def is_stream_consumed(self):
        pass

    @property
    def reason(self):
        pass

    @property
    def request(self):
        pass

    @property
    def status_code(self):
        pass

    @property
    def url(self):
        pass



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Contract tests: stream events survive terminal state and respect a 10-minute TTL.

These tests validate two critical invariants:

1. **Stream persistence after terminal state** — Once a bg+stream response
   reaches terminal status (completed, failed, etc.) and the in-memory
   execution record is eagerly evicted, the persisted SSE events MUST still
   be replayable via ``GET /responses/{id}?stream=true``.  This holds for
   both the default in-memory provider path and the Foundry-like hosted path
   (where the response provider does not implement ``ResponseStreamProviderProtocol``).

2. **Per-event 10-minute TTL (B35)** — Each SSE event carries a ``_saved_at``
   timestamp.  ``get_stream_events()`` filters out events older than the
   replay TTL (default 600 s / 10 minutes).  Events within the window MUST
   be returned; events outside the window MUST be filtered.
"""

from __future__ import annotations

import json
from datetime import datetime, timedelta, timezone
from typing import Any, Iterable

import pytest
from starlette.testclient import TestClient

from azure.ai.agentserver.responses import ResponsesAgentServerHost
from azure.ai.agentserver.responses.models._generated import OutputItem, ResponseObject
from azure.ai.agentserver.responses.store._base import (
    ResponseProviderProtocol,
    ResponseStreamProviderProtocol,
)
from azure.ai.agentserver.responses.store._memory import InMemoryResponseProvider
from azure.ai.agentserver.responses.streaming import ResponseEventStream
from tests._helpers import poll_until

# ────────────────────────────────────────
# Facade that strips stream capability (simulates Foundry / hosted)
# ────────────────────────────────────────


class _ResponseOnlyProvider:
    """Wraps InMemoryResponseProvider, exposing only ResponseProviderProtocol."""

    def __init__(self) -> None:
        self._inner = InMemoryResponseProvider()

    async def create_response(
        self,
        response: ResponseObject,
        input_items: Iterable[OutputItem] | None,
        history_item_ids: Iterable[str] | None,
        *,
        isolation: Any = None,
    ) -> None:
        await self._inner.create_response(response, input_items, history_item_ids, isolation=isolation)

    async def get_response(self, response_id: str, *, isolation: Any = None) -> ResponseObject:
        return await self._inner.get_response(response_id, isolation=isolation)

    async def update_response(self, response: ResponseObject, *, isolation: Any = None) -> None:
        await self._inner.update_response(response, isolation=isolation)

    async def delete_response(self, response_id: str, *, isolation: Any = None) -> None:
        await self._inner.delete_response(response_id, isolation=isolation)

    async def get_input_items(
        self,
        response_id: str,
        limit: int = 20,
        ascending: bool = False,
        after: str | None = None,
        before: str | None = None,
        *,
        isolation: Any = None,
    ) -> list[OutputItem]:
        return await self._inner.get_input_items(response_id, limit, ascending, after, before, isolation=isolation)

    async def get_items(
        self,
        item_ids: Iterable[str],
        *,
        isolation: Any = None,
    ) -> list[OutputItem | None]:
        return await self._inner.get_items(item_ids, isolation=isolation)

    async def get_history_item_ids(
        self,
        previous_response_id: str | None,
        conversation_id: str | None,
        limit: int,
        *,
        isolation: Any = None,
    ) -> list[str]:
        return await self._inner.get_history_item_ids(previous_response_id, conversation_id, limit, isolation=isolation)


# ────────────────────────────────────────
# Helpers
# ────────────────────────────────────────


def _build_client_default(handler: Any) -> TestClient:
    """Build a TestClient with the default InMemoryResponseProvider (non-hosted)."""
    app = ResponsesAgentServerHost()
    app.response_handler(handler)
    return TestClient(app)


def _build_client_hosted(handler: Any) -> TestClient:
    """Build a TestClient with a response-only provider (simulates Foundry / hosted)."""
    provider = _ResponseOnlyProvider()
    assert isinstance(provider, ResponseProviderProtocol)
    assert not isinstance(provider, ResponseStreamProviderProtocol)
    app = ResponsesAgentServerHost(store=provider)
    app.response_handler(handler)
    return TestClient(app)


def _handler(request: Any, context: Any, cancel: Any) -> Any:
    """Minimal handler: created → completed."""

    async def _events():
        stream = ResponseEventStream(
            response_id=context.response_id,
            model=getattr(request, "model", None),
        )
        yield stream.emit_created()
        yield stream.emit_completed()

    return _events()


def _handler_with_output(request: Any, context: Any, cancel: Any) -> Any:
    """Realistic handler: created → in_progress → message with text → completed."""

    async def _events():
        stream = ResponseEventStream(
            response_id=context.response_id,
            model=getattr(request, "model", None),
        )
        yield stream.emit_created()
        yield stream.emit_in_progress()

        message = stream.add_output_item_message()
        yield message.emit_added()
        text = message.add_text_content()
        yield text.emit_added()
        yield text.emit_delta("Hello from the agent.")
        yield text.emit_text_done("Hello from the agent.")
        yield text.emit_done()
        yield message.emit_done()
        yield stream.emit_completed()

    return _events()


def _collect_sse_events(response: Any) -> list[dict[str, Any]]:
    events: list[dict[str, Any]] = []
    current_type: str | None = None
    current_data: str | None = None
    for line in response.iter_lines():
        if not line:
            if current_type is not None:
                payload = json.loads(current_data) if current_data else {}
                events.append({"type": current_type, "data": payload})
            current_type = None
            current_data = None
            continue
        if line.startswith("event:"):
            current_type = line.split(":", 1)[1].strip()
        elif line.startswith("data:"):
            current_data = line.split(":", 1)[1].strip()
    return events


def _wait_for_terminal(client: TestClient, response_id: str) -> dict[str, Any]:
    latest: dict[str, Any] = {}

    def _is_terminal() -> bool:
        nonlocal latest
        r = client.get(f"/responses/{response_id}")
        if r.status_code != 200:
            return False
        latest = r.json()
        return latest.get("status") in {"completed", "failed", "incomplete", "cancelled"}

    ok, failure = poll_until(
        _is_terminal,
        timeout_s=5.0,
        interval_s=0.05,
        context_provider=lambda: {"status": latest.get("status")},
        label=f"wait_for_terminal({response_id})",
    )
    assert ok, failure
    return latest


def _create_bg_stream(client: TestClient) -> tuple[str, list[dict[str, Any]]]:
    """POST bg+stream, consume SSE, return (response_id, events)."""
    payload = {
        "model": "gpt-4o-mini",
        "input": "hello",
        "stream": True,
        "store": True,
        "background": True,
    }
    with client.stream("POST", "/responses", json=payload) as resp:
        assert resp.status_code == 200
        events = _collect_sse_events(resp)
    assert events, "Expected at least one SSE event"
    response_id = events[0]["data"]["response"]["id"]
    return response_id, events


# ════════════════════════════════════════════════════════════
# Tests: Stream survives terminal state
# ════════════════════════════════════════════════════════════


class TestStreamSurvivesTerminalState:
    """After terminal state + eager eviction, stream events must still be replayable."""

    def test_stream_replay_after_terminal_default_provider(self) -> None:
        """Default (non-hosted): stream replay works after completion + eviction."""
        client = _build_client_default(_handler)
        response_id, post_events = _create_bg_stream(client)
        snapshot = _wait_for_terminal(client, response_id)
        assert snapshot["status"] == "completed"

        # Runtime state should be evicted — GET falls through to provider
        # Stream replay must still work
        with client.stream("GET", f"/responses/{response_id}?stream=true") as replay:
            assert replay.status_code == 200
            replay_events = _collect_sse_events(replay)

        assert len(replay_events) >= 2, f"Expected >= 2 events, got {len(replay_events)}"
        event_types = [e["type"] for e in replay_events]
        assert "response.created" in event_types
        assert "response.completed" in event_types
        assert replay_events[-1]["type"] == "response.completed"

    def test_stream_replay_after_terminal_hosted_provider(self) -> None:
        """Hosted (Foundry-like): stream replay works after completion + eviction."""
        client = _build_client_hosted(_handler)
        response_id, post_events = _create_bg_stream(client)
        snapshot = _wait_for_terminal(client, response_id)
        assert snapshot["status"] == "completed"

        with client.stream("GET", f"/responses/{response_id}?stream=true") as replay:
            assert replay.status_code == 200
            replay_events = _collect_sse_events(replay)

        assert len(replay_events) >= 2
        event_types = [e["type"] for e in replay_events]
        assert "response.created" in event_types
        assert "response.completed" in event_types

    def test_stream_replay_preserves_all_events_default(self) -> None:
        """Default: all events from a realistic handler survive terminal + eviction."""
        client = _build_client_default(_handler_with_output)
        response_id, post_events = _create_bg_stream(client)
        _wait_for_terminal(client, response_id)

        with client.stream("GET", f"/responses/{response_id}?stream=true") as replay:
            assert replay.status_code == 200
            replay_events = _collect_sse_events(replay)

        # Realistic handler emits: created, in_progress, item.added, content.added,
        # content.delta, content.text_done, content.done, item.done, completed = 9
        assert len(replay_events) >= 5, f"Expected >= 5 events, got {len(replay_events)}"
        replay_types = [e["type"] for e in replay_events]
        assert replay_types[0] == "response.created"
        assert replay_types[-1] == "response.completed"
        assert "response.output_item.added" in replay_types

    def test_stream_replay_preserves_all_events_hosted(self) -> None:
        """Hosted: all events from a realistic handler survive terminal + eviction."""
        client = _build_client_hosted(_handler_with_output)
        response_id, post_events = _create_bg_stream(client)
        _wait_for_terminal(client, response_id)

        with client.stream("GET", f"/responses/{response_id}?stream=true") as replay:
            assert replay.status_code == 200
            replay_events = _collect_sse_events(replay)

        assert len(replay_events) >= 5, f"Expected >= 5 events, got {len(replay_events)}"
        replay_types = [e["type"] for e in replay_events]
        assert replay_types[0] == "response.created"
        assert replay_types[-1] == "response.completed"
        assert "response.output_item.added" in replay_types

    def test_multiple_replays_after_terminal(self) -> None:
        """Stream can be replayed multiple times after terminal state."""
        client = _build_client_default(_handler)
        response_id, _ = _create_bg_stream(client)
        _wait_for_terminal(client, response_id)

        for _ in range(3):
            with client.stream("GET", f"/responses/{response_id}?stream=true") as replay:
                assert replay.status_code == 200
                events = _collect_sse_events(replay)
            assert len(events) >= 2

    def test_multiple_replays_after_terminal_hosted(self) -> None:
        """Hosted: stream can be replayed multiple times after terminal state."""
        client = _build_client_hosted(_handler)
        response_id, _ = _create_bg_stream(client)
        _wait_for_terminal(client, response_id)

        for _ in range(3):
            with client.stream("GET", f"/responses/{response_id}?stream=true") as replay:
                assert replay.status_code == 200
                events = _collect_sse_events(replay)
            assert len(events) >= 2


# ════════════════════════════════════════════════════════════
# Tests: Per-event 10-minute TTL (B35)
# ════════════════════════════════════════════════════════════


class TestStreamEventTTL:
    """Each stream event must be replayable for 10 minutes after emission, then filtered."""

    @pytest.mark.asyncio
    async def test_events_within_ttl_are_returned(self) -> None:
        """Events saved less than 10 minutes ago are returned by get_stream_events."""
        provider = InMemoryResponseProvider()
        rid = "caresp_ttl_within_0000000000000000"
        now = datetime.now(timezone.utc)

        events = [
            {"type": "response.created", "_saved_at": now - timedelta(minutes=5)},
            {"type": "response.completed", "_saved_at": now - timedelta(minutes=3)},
        ]
        await provider.save_stream_events(rid, events)

        result = await provider.get_stream_events(rid)
        assert result is not None
        assert len(result) == 2
        assert result[0]["type"] == "response.created"
        assert result[1]["type"] == "response.completed"

    @pytest.mark.asyncio
    async def test_events_older_than_10_minutes_are_filtered(self) -> None:
        """Events saved more than 10 minutes ago are filtered or purged entirely."""
        provider = InMemoryResponseProvider()
        rid = "caresp_ttl_exact_0000000000000000"
        now = datetime.now(timezone.utc)

        events = [
            {"type": "response.created", "_saved_at": now - timedelta(minutes=11)},
            {"type": "response.completed", "_saved_at": now - timedelta(minutes=11)},
        ]
        await provider.save_stream_events(rid, events)

        result = await provider.get_stream_events(rid)
        # Either None (purged entirely by orphan cleanup) or empty list
        if result is not None:
            assert len(result) == 0, "Events older than 10 min should be filtered"

    @pytest.mark.asyncio
    async def test_events_well_past_ttl_are_gone(self) -> None:
        """Events saved well beyond the 10-minute TTL must be filtered or purged."""
        provider = InMemoryResponseProvider()
        rid = "caresp_ttl_old_000000000000000000"
        now = datetime.now(timezone.utc)

        events = [
            {"type": "response.created", "_saved_at": now - timedelta(minutes=15)},
            {"type": "response.completed", "_saved_at": now - timedelta(minutes=12)},
        ]
        await provider.save_stream_events(rid, events)

        result = await provider.get_stream_events(rid)
        # Either None (purged entirely by orphan cleanup) or empty list
        if result is not None:
            assert len(result) == 0, "All events older than 10 min should be filtered"

    @pytest.mark.asyncio
    async def test_mixed_ttl_only_live_events_returned(self) -> None:
        """Only events within the 10-minute window survive; older ones are dropped."""
        provider = InMemoryResponseProvider()
        rid = "caresp_ttl_mixed_0000000000000000"
        now = datetime.now(timezone.utc)

        events = [
            {"type": "response.created", "_saved_at": now - timedelta(minutes=12)},
            {"type": "response.in_progress", "_saved_at": now - timedelta(minutes=8)},
            {"type": "response.output_item.added", "_saved_at": now - timedelta(minutes=5)},
            {"type": "response.completed", "_saved_at": now - timedelta(minutes=2)},
        ]
        await provider.save_stream_events(rid, events)

        result = await provider.get_stream_events(rid)
        assert result is not None
        assert len(result) == 3, f"Expected 3 live events, got {len(result)}"
        types = [e["type"] for e in result]
        assert "response.created" not in types, "12-min-old event should be filtered"
        assert "response.in_progress" in types
        assert "response.output_item.added" in types
        assert "response.completed" in types

    @pytest.mark.asyncio
    async def test_events_just_under_10_minutes_survive(self) -> None:
        """Events saved 9 minutes 59 seconds ago are still within the TTL window."""
        provider = InMemoryResponseProvider()
        rid = "caresp_ttl_just_000000000000000000"
        now = datetime.now(timezone.utc)

        events = [
            {"type": "response.created", "_saved_at": now - timedelta(minutes=9, seconds=59)},
            {"type": "response.completed", "_saved_at": now - timedelta(minutes=9, seconds=59)},
        ]
        await provider.save_stream_events(rid, events)

        result = await provider.get_stream_events(rid)
        assert result is not None
        assert len(result) == 2, "Events at 9m59s should still be within TTL"

    @pytest.mark.asyncio
    async def test_orphaned_stream_events_purged_after_ttl(self) -> None:
        """Standalone stream-only usage: purge removes events older than TTL.

        When InMemoryResponseProvider is used as a fallback stream provider
        (no _entries for those response IDs), purge_expired must still clean
        up stream events whose _saved_at exceeds the replay TTL.
        """
        provider = InMemoryResponseProvider()
        rid = "caresp_ttl_orphan_00000000000000000"
        old_time = datetime.now(timezone.utc) - timedelta(minutes=15)

        events = [
            {"type": "response.created", "_saved_at": old_time},
            {"type": "response.completed", "_saved_at": old_time},
        ]
        await provider.save_stream_events(rid, events)

        # The auto-purge on each _locked() call cleans orphaned stale events.
        # After saving stale events and then reading, the stale events are
        # either filtered on read or purged entirely by the orphan cleanup.
        result = await provider.get_stream_events(rid)
        # Result is None (purged) or empty (filtered) — either way, no events.
        if result is not None:
            assert len(result) == 0, "Stale events should be filtered"

        # Explicitly call purge_expired to confirm cleanup
        await provider.purge_expired()

        # After explicit purge, the key must be gone entirely
        after_purge = await provider.get_stream_events(rid)
        # The key was already removed; should be None
        assert after_purge is None, "Orphaned stream events should be fully purged after TTL"



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
This is a sample Azure Function app used by sample_agent_azure_function.py
to demonstrate how to integrate Azure Functions with AI Agents using queue triggers.
"""

# import azure.functions as func
# import logging
# import json

# app = func.FunctionApp()

# @app.queue_trigger(arg_name="msg", queue_name="get-weather-input-queue", connection="STORAGE_CONNECTION")
# @app.queue_output(arg_name="outputQueue", queue_name="get-weather-output-queue", connection="STORAGE_CONNECTION")
# def queue_trigger(msg: func.QueueMessage, outputQueue: func.Out[str]):
#     try:
#         messagepayload: dict = json.loads(msg.get_body().decode("utf-8"))
#         logging.info(f"The function receives the following message: {json.dumps(messagepayload)}")

#         function_args: dict = messagepayload.get("function_args", {})
#         location: str = function_args.get("location")


#         weather_result = f"Weather is {len(location)} degrees and sunny in {location}"
#         response_message = {
#             "Value": weather_result,
#             "CorrelationId": messagepayload["CorrelationId"]
#         }
#         logging.info(f"The function returns the following message through the {outputQueue} queue: {json.dumps(response_message)}")

#         outputQueue.set(json.dumps(response_message))

#     except Exception as e:
#         logging.error(f"Error processing message: {e}")



# coding=utf-8

from enum import Enum
from corehttp.utils import CaseInsensitiveEnumMeta


class DaysOfWeekExtensibleEnum(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Days of the week."""

    MONDAY = "Monday"
    """Monday."""
    TUESDAY = "Tuesday"
    """Tuesday."""
    WEDNESDAY = "Wednesday"
    """Wednesday."""
    THURSDAY = "Thursday"
    """Thursday."""
    FRIDAY = "Friday"
    """Friday."""
    SATURDAY = "Saturday"
    """Saturday."""
    SUNDAY = "Sunday"
    """Sunday."""



"""Swagger to SDK"""

import os
import shutil
import logging
import json
from pathlib import Path
import tempfile

from git import Repo, GitCommandError

from .SwaggerToSdkCore import (
    read_config_from_github,
    DEFAULT_COMMIT_MESSAGE,
    get_input_paths,
    extract_conf_from_readmes,
    get_readme_files_from_git_object,
    build_file_content,
    solve_relative_path,
    this_conf_will_generate_for_this_pr,
)
from .autorest_tools import (
    execute_simple_command,
    generate_code,
    merge_options,
)
from ci_tools.git_tools import checkout_and_create_branch, do_commit
from ci_tools.github_tools import configure_user, manage_git_folder


_LOGGER = logging.getLogger(__name__)


def move_wrapper_files_or_dirs(src_root, dst_root, global_conf, local_conf):
    """Save wrapper files somewhere for replace them after generation."""
    src_relative_path = local_conf.get("output_dir", "")
    src_abs_path = Path(src_root, src_relative_path)
    dst_abs_path = Path(dst_root, src_relative_path)

    wrapper_files_or_dirs = merge_options(global_conf, local_conf, "wrapper_filesOrDirs") or []

    for wrapper_file_or_dir in wrapper_files_or_dirs:
        for file_path in src_abs_path.glob(wrapper_file_or_dir):
            relative_file_path = file_path.relative_to(src_abs_path)
            file_path_dest = Path(dst_abs_path, relative_file_path)
            if file_path.is_file():
                file_path_dest.parent.mkdir(parents=True, exist_ok=True)
            _LOGGER.info("Moving %s to %s", str(file_path), str(file_path_dest))
            # This does not work in Windows if generatd and dest are not in the same drive
            # file_path.replace(file_path_dest)
            shutil.move(file_path, file_path_dest)


def delete_extra_files(sdk_root, global_conf, local_conf):
    src_relative_path = local_conf.get("output_dir", "")
    src_abs_path = Path(sdk_root, src_relative_path)

    delete_files_or_dirs = merge_options(global_conf, local_conf, "delete_filesOrDirs") or []

    for delete_file_or_dir in delete_files_or_dirs:
        for file_path in src_abs_path.glob(delete_file_or_dir):
            if file_path.is_file():
                file_path.unlink()
            else:
                shutil.rmtree(str(file_path))


def move_autorest_files(client_generated_path, sdk_root, global_conf, local_conf):
    """Update data from generated to final folder.

    This is one only if output_dir is set, otherwise it's considered generated in place
    and does not required moving
    """
    dest = local_conf.get("output_dir", None)
    if not dest:
        return
    destination_folder = get_local_path_dir(sdk_root, dest)

    generated_relative_base_directory = local_conf.get("generated_relative_base_directory") or global_conf.get(
        "generated_relative_base_directory"
    )

    if generated_relative_base_directory:
        client_possible_path = [
            elt for elt in client_generated_path.glob(generated_relative_base_directory) if elt.is_dir()
        ]
        try:
            client_generated_path = client_possible_path.pop()
        except IndexError:
            err_msg = "Incorrect generated_relative_base_directory folder: {}\n".format(
                generated_relative_base_directory
            )
            err_msg += "Base folders were: : {}\n".format(
                [f.relative_to(client_generated_path) for f in client_generated_path.iterdir()]
            )
            _LOGGER.critical(err_msg)
            raise ValueError(err_msg)
        if client_possible_path:
            err_msg = "generated_relative_base_directory parameter is ambiguous: {} {}".format(
                client_generated_path, client_possible_path
            )
            _LOGGER.critical(err_msg)
            raise ValueError(err_msg)

    shutil.rmtree(str(destination_folder))
    # This does not work in Windows if generatd and dest are not in the same drive
    # client_generated_path.replace(destination_folder)
    shutil.move(client_generated_path, destination_folder)


def write_build_file(sdk_root, local_conf):
    build_dir = local_conf.get("build_dir")
    if build_dir:
        build_folder = get_local_path_dir(sdk_root, build_dir)
        build_file = Path(build_folder, "build.json")
        with open(build_file, "w") as build_fd:
            json.dump(build_file_content(), build_fd, indent=2)


def execute_after_script(sdk_root, global_conf, local_conf):
    after_scripts = merge_options(global_conf, local_conf, "after_scripts", keep_list_order=True) or []
    local_envs = dict(os.environ)
    local_envs.update(global_conf.get("envs", {}))

    for script in after_scripts:
        _LOGGER.info("Execute after script: %s", script)
        execute_simple_command(script, cwd=sdk_root, shell=True, env=local_envs)


def get_local_path_dir(root, relative_path):
    build_folder = Path(root, relative_path)
    if not build_folder.is_dir():
        err_msg = "Folder does not exist or is not accessible: {}".format(build_folder)
        _LOGGER.critical(err_msg)
        raise ValueError(err_msg)
    return build_folder


def build_project(
    temp_dir,
    project,
    absolute_markdown_path,
    sdk_folder,
    global_conf,
    local_conf,
    autorest_bin=None,
):
    absolute_generated_path = Path(temp_dir, project)
    absolute_save_path = Path(temp_dir, "save")
    move_wrapper_files_or_dirs(sdk_folder, absolute_save_path, global_conf, local_conf)
    generate_code(
        absolute_markdown_path,
        global_conf,
        local_conf,
        absolute_generated_path if "output_dir" in local_conf else None,
        autorest_bin,
    )
    move_autorest_files(absolute_generated_path, sdk_folder, global_conf, local_conf)
    move_wrapper_files_or_dirs(absolute_save_path, sdk_folder, global_conf, local_conf)
    delete_extra_files(sdk_folder, global_conf, local_conf)
    write_build_file(sdk_folder, local_conf)
    execute_after_script(sdk_folder, global_conf, local_conf)


def build_libraries(config, skip_callback, restapi_git_folder, sdk_repo, temp_dir, autorest_bin=None):
    """Main method of the the file"""

    global_conf = config["meta"]
    global_conf["autorest_options"] = solve_relative_path(
        global_conf.get("autorest_options", {}), sdk_repo.working_tree_dir
    )
    global_conf["envs"] = solve_relative_path(global_conf.get("envs", {}), sdk_repo.working_tree_dir)
    global_conf["advanced_options"] = solve_relative_path(
        global_conf.get("advanced_options", {}), sdk_repo.working_tree_dir
    )

    for project, local_conf in config.get("projects", {}).items():
        if skip_callback(project, local_conf):
            _LOGGER.info("Skip project %s", project)
            continue
        local_conf["autorest_options"] = solve_relative_path(
            local_conf.get("autorest_options", {}), sdk_repo.working_tree_dir
        )

        markdown_relative_path, optional_relative_paths = get_input_paths(global_conf, local_conf)
        _LOGGER.info(f"Markdown input: {markdown_relative_path}")
        _LOGGER.info(f"Optional inputs: {optional_relative_paths}")

        absolute_markdown_path = None
        if markdown_relative_path:
            absolute_markdown_path = Path(restapi_git_folder, markdown_relative_path).resolve()
        if optional_relative_paths:
            local_conf.setdefault("autorest_options", {})["input-file"] = [
                Path(restapi_git_folder, input_path).resolve() for input_path in optional_relative_paths
            ]

        sdk_folder = sdk_repo.working_tree_dir
        build_project(
            temp_dir,
            project,
            absolute_markdown_path,
            sdk_folder,
            global_conf,
            local_conf,
            autorest_bin,
        )


def generate_sdk_from_git_object(
    git_object,
    branch_name,
    restapi_git_id,
    sdk_git_id,
    base_branch_names,
    *,
    fallback_base_branch_name="main",
    sdk_tag=None,
):
    """Generate SDK from a commit or a PR object.

    git_object is the initial commit/PR from the RestAPI repo. If git_object is a PR, prefer to checkout Github PR "merge_commit_sha"
    restapi_git_id explains where to clone the repo.
    sdk_git_id explains where to push the commit.
    sdk_tag explains what is the tag used in the Readme for the swagger-to-sdk section. If not provided, use sdk_git_id.
    branch_name is the expected branch name in the SDK repo.
    - If this branch exists, use it.
    - If not, use the base branch to create that branch (base branch is where I intend to do my PR)
    - If base_branch_names is not provided, use fallback_base_branch_name as base
    - If this base branch is provided and does not exists, create this base branch first using fallback_base_branch_name (this one is required to exist)

    WARNING:
    This method might push to "branch_name" and "base_branch_name". No push will be made to "fallback_base_branch_name"
    """
    gh_token = os.environ["GH_TOKEN"]
    message_template = DEFAULT_COMMIT_MESSAGE
    autorest_bin = None
    if sdk_tag is None:
        sdk_tag = sdk_git_id

    try:  # Checkout the sha if commit obj
        branched_rest_api_id = restapi_git_id + "@" + git_object.sha
        pr_number = None
    except (
        AttributeError,
        TypeError,
    ):  # This is a PR, don't clone the fork but "base" repo and PR magic commit
        if git_object.merge_commit_sha:
            branched_rest_api_id = git_object.base.repo.full_name + "@" + git_object.merge_commit_sha
        else:
            branched_rest_api_id = git_object.base.repo.full_name
        pr_number = git_object.number

    # Always clone SDK from fallback branch that is required to exist
    branched_sdk_git_id = sdk_git_id + "@" + fallback_base_branch_name

    # I don't know if the destination branch exists, try until it works
    config = None
    branch_list = base_branch_names + [branch_name] + [fallback_base_branch_name]
    for branch in branch_list:
        try:
            config = read_config_from_github(sdk_git_id, branch, gh_token)
        except Exception:
            pass
        else:
            break
    if config is None:
        raise ValueError("Unable to locate configuration in {}".format(branch_list))
    global_conf = config["meta"]

    # If PR is only about a language that this conf can't handle, skip fast
    if not this_conf_will_generate_for_this_pr(git_object, global_conf):
        _LOGGER.info("Skipping this job based on conf not impacted by Git object")
        return

    with tempfile.TemporaryDirectory() as temp_dir:

        clone_dir = Path(temp_dir) / Path(global_conf.get("advanced_options", {}).get("clone_dir", "sdk"))
        _LOGGER.info("Clone dir will be: %s", clone_dir)

        with manage_git_folder(
            gh_token,
            Path(temp_dir) / Path("rest"),
            branched_rest_api_id,
            pr_number=pr_number,
        ) as restapi_git_folder, manage_git_folder(gh_token, clone_dir, branched_sdk_git_id) as sdk_folder:

            readme_files_infered = get_readme_files_from_git_object(git_object, restapi_git_folder)
            _LOGGER.info("Readmes files infered from PR: %s ", readme_files_infered)
            if not readme_files_infered:
                _LOGGER.info("No Readme in PR, quit")
                return

            # SDK part
            sdk_repo = Repo(str(sdk_folder))

            for base_branch in base_branch_names:
                _LOGGER.info("Checkout and create %s", base_branch)
                checkout_and_create_branch(sdk_repo, base_branch)

            _LOGGER.info("Try to checkout destination branch %s", branch_name)
            try:
                sdk_repo.git.checkout(branch_name)
                _LOGGER.info("The branch exists.")
            except GitCommandError:
                _LOGGER.info("Destination branch does not exists")
                # Will be created by do_commit

            configure_user(gh_token, sdk_repo)

            # Look for configuration in Readme
            _LOGGER.info("Extract conf from Readmes for target: %s", sdk_git_id)
            extract_conf_from_readmes(readme_files_infered, restapi_git_folder, sdk_tag, config)
            _LOGGER.info("End of extraction")

            def skip_callback(project, local_conf):
                # We know "project" is based on Path in "readme_files_infered"
                if Path(project) in readme_files_infered:
                    return False
                # Might be a regular project
                markdown_relative_path, optional_relative_paths = get_input_paths(global_conf, local_conf)
                if not (
                    markdown_relative_path in readme_files_infered
                    or any(input_file in readme_files_infered for input_file in optional_relative_paths)
                ):
                    _LOGGER.info(f"In project {project} no files involved in this commit")
                    return True
                return False

            build_libraries(
                config,
                skip_callback,
                restapi_git_folder,
                sdk_repo,
                temp_dir,
                autorest_bin,
            )

            try:
                commit_for_sha = git_object.commit  # Commit
            except AttributeError:
                commit_for_sha = list(git_object.get_commits())[-1].commit  # PR
            message = message_template + "\n\n" + commit_for_sha.message
            commit_sha = do_commit(sdk_repo, message, branch_name, commit_for_sha.sha)
            if commit_sha:
                for base_branch in base_branch_names:
                    sdk_repo.git.push("origin", base_branch, set_upstream=True)
                sdk_repo.git.push("origin", branch_name, set_upstream=True)
                return "https://github.com/{}/commit/{}".format(sdk_git_id, commit_sha)



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_text_abstractive_summarization.py

DESCRIPTION:
    This sample demonstrates how to run an **abstractive summarization** action over text.

USAGE:
    python sample_text_abstractive_summarization.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_TEXT_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_TEXT_ENDPOINT
      - AZURE_TEXT_KEY
"""

# [START text_abstractive_summarization]
import os

from azure.identity import DefaultAzureCredential
from azure.ai.textanalytics import TextAnalysisClient
from azure.ai.textanalytics.models import (
    MultiLanguageTextInput,
    MultiLanguageInput,
    AbstractiveSummarizationOperationAction,
    AbstractiveSummarizationOperationResult,
)


def sample_text_abstractive_summarization():
    # get settings
    endpoint = os.environ["AZURE_TEXT_ENDPOINT"]
    credential = DefaultAzureCredential()

    client = TextAnalysisClient(endpoint, credential=credential)

    # Build input
    text_a = (
        "Windows 365 was in the works before COVID-19 sent companies around the world on a scramble to secure "
        "solutions to support employees suddenly forced to work from home, but “what really put the "
        "firecracker behind it was the pandemic, it accelerated everything,” McKelvey said. She explained "
        "that customers were asking, “How do we create an experience for people that makes them still feel "
        "connected to the company without the physical presence of being there?” In this new world of "
        "Windows 365, remote workers flip the lid on their laptop, boot up the family workstation or clip a "
        "keyboard onto a tablet, launch a native app or modern web browser and login to their Windows 365 "
        "account. From there, their Cloud PC appears with their background, apps, settings and content just "
        "as they left it when they last were last there – in the office, at home or a coffee shop. And "
        "then, when you’re done, you’re done. You won’t have any issues around security because you’re not "
        "saving anything on your device,” McKelvey said, noting that all the data is stored in the cloud. "
        "The ability to login to a Cloud PC from anywhere on any device is part of Microsoft’s larger "
        "strategy around tailoring products such as Microsoft Teams and Microsoft 365 for the post-pandemic "
        "hybrid workforce of the future, she added. It enables employees accustomed to working from home to "
        "continue working from home; it enables companies to hire interns from halfway around the world; it "
        "allows startups to scale without requiring IT expertise. “I think this will be interesting for "
        "those organizations who, for whatever reason, have shied away from virtualization. This is giving "
        "them an opportunity to try it in a way that their regular, everyday endpoint admin could manage,” "
        "McKelvey said. The simplicity of Windows 365 won over Dean Wells, the corporate chief information "
        "officer for the Government of Nunavut. His team previously attempted to deploy a traditional "
        "virtual desktop infrastructure and found it inefficient and unsustainable given the limitations of "
        "low-bandwidth satellite internet and the constant need for IT staff to manage the network and "
        "infrastructure. We didn’t run it for very long,” he said. “It didn’t turn out the way we had "
        "hoped. So, we actually had terminated the project and rolled back out to just regular PCs.” He "
        "re-evaluated this decision after the Government of Nunavut was hit by a ransomware attack in "
        "November 2019 that took down everything from the phone system to the government’s servers. "
        "Microsoft helped rebuild the system, moving the government to Teams, SharePoint, OneDrive and "
        "Microsoft 365. Manchester’s team recruited the Government of Nunavut to pilot Windows 365. Wells "
        "was intrigued, especially by the ability to manage the elastic workforce securely and seamlessly. "
        "“The impact that I believe we are finding, and the impact that we’re going to find going forward, "
        "is being able to access specialists from outside the territory and organizations outside the "
        "territory to come in and help us with our projects, being able to get people on staff with us to "
        "help us deliver the day-to-day expertise that we need to run the government,” he said. “Being able "
        "to improve healthcare, being able to improve education, economic development is going to improve "
        "the quality of life in the communities.”"
    )

    text_input = MultiLanguageTextInput(multi_language_inputs=[MultiLanguageInput(id="A", text=text_a, language="en")])

    action = AbstractiveSummarizationOperationAction(name="Abstractive Summarization")

    # Start long-running operation (sync)
    poller = client.begin_analyze_text_job(
        text_input=text_input,
        actions=[action],
    )

    # Operation metadata (pre-final)
    print(f"Operation ID: {poller.details.get('operation_id')}")

    # Wait for completion and get pageable of TextActions
    paged_actions = poller.result()

    # Final-state metadata
    d = poller.details
    print(f"Job ID: {d.get('job_id')}")
    print(f"Status: {d.get('status')}")
    print(f"Created: {d.get('created_date_time')}")
    print(f"Last Updated: {d.get('last_updated_date_time')}")
    if d.get("expiration_date_time"):
        print(f"Expires: {d.get('expiration_date_time')}")
    if d.get("display_name"):
        print(f"Display Name: {d.get('display_name')}")

    # Iterate results (sync pageable)
    for actions_page in paged_actions:
        # Page-level counts if available on TextActions
        print(
            f"Completed: {actions_page.completed}, "
            f"In Progress: {actions_page.in_progress}, "
            f"Failed: {actions_page.failed}, "
            f"Total: {actions_page.total}"
        )

        # Items are the individual operation results
        for op_result in actions_page.items_property or []:
            if isinstance(op_result, AbstractiveSummarizationOperationResult):
                print(f"\nAction Name: {op_result.task_name}")
                print(f"Action Status: {op_result.status}")
                print(f"Kind: {op_result.kind}")

                # Documents summarized by this action
                results = op_result.results
                for doc in results.documents or []:
                    print(f"\nDocument ID: {doc.id}")
                    for s in doc.summaries or []:
                        print(f"  Summary: {s.text}")
            else:
                # Other action kinds, if present
                try:
                    print(
                        f"\n[Non-abstractive action] name={op_result.task_name}, "
                        f"status={op_result.status}, kind={op_result.kind}"
                    )
                except (AttributeError, TypeError) as e:
                    print(f"\n[Non-abstractive action present] Error: {e}")


# [END text_abstractive_summarization]


def main():
    sample_text_abstractive_summarization()


if __name__ == "__main__":
    main()
